"""
Module to parse output from 'ioreg' command to python data structures
"""

import sys
import os
import re
from subprocess import Popen, PIPE

IOREG_COMMAND = '/usr/sbin/ioreg'

RE_IOREG_HEADER = re.compile('^\+-o\s(?P<name>.*)\s+<class (?P<ioclass>[\w]+),(?P<flags>[^>]*)')

class IORegError(Exception):
    """
    Execptions raised by ioreg command output parsers.
    """
    def __str__(self):
        return self.args[0]

class IORegItem(object):
    """
    One information item line from ioreg command output
    """
    def __init__(self, line):
        try:
            key, value = line.split('=', 1)
        except ValueError:
            try:
                key, value = line.split(':', 1)
            except ValueError, emsg:
                raise IORegError('Error splitting item line %s' % line)

        self.key = key.rstrip().strip('"')
        self.value = value.strip()

    def __repr__(self):
        return '%s: %s' % (self.key, self.value)

class IORegGroup(dict):
    """
    A group of items in ioreg command output
    """
    def __init__(self, parent, header):
        self.parent = parent
        self.name = 'UNPARSED'

        m = RE_IOREG_HEADER.match(header)
        if not m:
            raise IORegError('Error parsing header: %s' % header)

        for k, v in m.groupdict().items():
            setattr(self, k, v.strip())

    def __repr__(self):
        return 'IORegGroup %s' % self.name

    def append(self, line):
        """
        Add an IORegItem entry to this group
        """
        line = line.strip()
        if line in ['"', '']:
            return
        item = IORegItem(line)
        self[item.key] = item

class IORegTree(list):
    """
    Parser for ioreg output entries to a dictionary
    """
    def __init__(self, path=None):
        if not os.access(IOREG_COMMAND, os.X_OK):
            raise IORegError('Not executable: %s' % IOREG_COMMAND)

        if path is not None:
            cmd = [IOREG_COMMAND, '-r', '-w0', '-n', path]
        else:
            cmd = [IOREG_COMMAND, '-lw0']

        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        stdout, stderr = p.communicate()

        parent = None
        group = None
        entries = []
        for l in [l.lstrip(' |').rstrip() for l in stdout.split('\n')]:
            if l in ['', '{']:
                continue
            elif l[:3] == '+-o':
                group = IORegGroup(parent, l)
                parent = group
                self.append(group)

            elif l == '}':
                group = None
            elif group is not None:
                group.append(l)
            else:
                print 'ENTRY out of group: %s' % l




#!/usr/bin/env python
"""
Send notifications to OS/X Mountain Lion notification center
"""

import Foundation
import AppKit
import objc
import time
from PyObjCTools import AppHelper

class NotificationClient(Foundation.NSObject):
    def __init__(self,*args,**kwargs):
        Foundation.NSObject.__init__(self,*args,**kwargs)

    def notify(self,title,subtitle,text,url=None):
        ncenter = objc.lookUpClass('NSUserNotificationCenter')
        nclass = objc.lookUpClass('NSUserNotification')
        user_notifications = ncenter.defaultUserNotificationCenter()
        user_notifications.setDelegate_(self)
        notification =  nclass.alloc().init()
        notification.setTitle_(str(title))
        notification.setSubtitle_(str(subtitle))
        notification.setInformativeText_(str(text))
        notification.setSoundName_("NSUserNotificationDefaultSoundName")
        notification.setHasActionButton_(False)
        notification.setOtherButtonTitle_("View")
        if url is not None:
            notification.setUserInfo_({"action":"open_url", "value":url})
        user_notifications.scheduleNotification_(notification)

    def userNotificationCenter_didActivateNotification_(self,center,notification):
        userInfo = notification.userInfo()
        if userInfo["action"] == "open_url":
            import subprocess
            subprocess.Popen(['open', userInfo["value"]])



#!/usr/bin/env python
"""
Abstraction of filesystem mount points for OS/X
"""
import os
import re
from subprocess import check_output, CalledProcessError
from mactypes import Alias

from systematic.filesystems import MountPoint, FileSystemError
from darwinist.diskutil import DiskInfo, DiskUtilError

re_mountpoint = re.compile(r'([^\s]*) on (.*) \(([^\)]*)\)$')
re_df = re.compile(r'^([^\s]+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)%\s+(.*)$')

class MountPoints(dict):
    """
    OS/X mount points parser
    """
    def __init__(self):
        self.update()

    def update(self):
        """
        Update mount points from /sbin/mount output
        """
        self.clear()
        try:
            output = check_output(['/sbin/mount'])
        except CalledProcessError,e:
            raise FileSystemError('Error getting mountpoints: %s' % e)

        for l in [l for l in output.split('\n') if l.strip()!='']:
            if l[:4] == 'map ':
                continue

            m = re_mountpoint.match(l)
            if not m:
                continue

            device = m.group(1)
            mountpoint = m.group(2)
            flags = map(lambda x: x.strip(), m.group(3).split(','))
            filesystem = flags[0]
            flags = flags[1:]

            entry = OSXMountPoint(mountpoint,device,filesystem)
            for f in flags:
                if f[:11] == 'mounted by ':
                    entry.flags.set('owner',f[11:])
                else:
                    entry.flags.set(f,True)
            self[entry.path] = entry
            

class OSXMountPoint(MountPoint):
    """
    One OS/X mountpoint parsed from /sbin/mount output

    Extra attributes:
    hfspath     Returns OS/X 'hfs path' or None
    """
    def __init__(self,mountpoint,device=None,filesystem=None):
        MountPoint.__init__(self,device,mountpoint,filesystem)
        try:
            self.hfspath = Alias(self.mountpoint).hfspath
        except ValueError:
            self.hfspath = None
        self.update_diskinfo()

    def update_diskinfo(self):
        if os.access(self.device,os.R_OK):
            self.diskinfo = DiskInfo(self.device)
        else:
            self.diskinfo = {}

    @property
    def name(self):
        return self.diskinfo.has_key('VolumeName') and self.diskinfo['VolumeName'] or os.path.basename(self.mountpoint)

    @property
    def size(self):
        try:
            return self.usage['size']
        except KeyError:
            return 0

    @property
    def used(self):
        try:
            return self.usage['used']
        except KeyError:
            return 0

    @property
    def available(self):
        try:
            return self.usage['available']
        except KeyError:
            return 0

    @property
    def percent(self):
        try:
            return self.usage['percent']
        except KeyError:
            return 0

    @property
    def writable(self):
        return self.diskinfo.has_key('Writable') and  self.diskinfo['Writable'] or False

    @property
    def writable(self):
        return self.diskinfo.has_key('Writable') and  self.diskinfo['Writable'] or False

    @property
    def bootable(self):
        return self.diskinfo.has_key('Bootable') and  self.diskinfo['Bootable'] or False

    @property
    def internal(self):
        return self.diskinfo.has_key('Internal') and  self.diskinfo['Internal'] or False

    @property
    def ejectable(self):
        return self.diskinfo.has_key('Ejectable') and  self.diskinfo['Ejectable'] or True

    @property
    def removable(self):
        return self.diskinfo.has_key('Removable') and  self.diskinfo['Removable'] or False

    @property
    def blocksize(self):
        return self.diskinfo.has_key('DeviceBlockSize') and  self.diskinfo['DeviceBlockSize'] or 0

    @property
    def usage(self):
        """
        Check usage percentage for this mountpoint.
        Returns dictionary with usage details.
        """
        try:
            output = check_output(['df','-k',self.mountpoint])
        except CalledProcessError,e:
            raise FileSystemError('Error checking filesystem usage: %s' % e)
        (header,usage) = output.split('\n',1)

        m = re_df.match(usage)
        if not m:
            raise FileSystemError('Error matching df output line: %s' % usage)
        return {
            'mountpoint': self.mountpoint,
            'size': long(m.group(2)),
            'used': long(m.group(3)),
            'free': long(m.group(4)),
            'percent': int(m.group(5))
        }




#!/usr/bin/env python
"""
OS/X application bundle processing tools
"""

import os
import plistlib

from xml.parsers.expat import ExpatError

INFO_BUNDLE_NAME_MAP = {
    'name':             'CFBundleName',
    'bundle_id':        'CFBundleIdentifier',
    'version':          'CFBundleVersion',
    'short_version':    'CFBundleShortVersionString',
}

class ApplicationError(Exception):
    pass

class Application(object):
    """
    Class to represent OS/X application bundles (.app)
    """
    def __init__(self,path):
        self.path = path
        self.__cached_info = None

        if not os.path.isdir(path):
            raise ApplicationError('No such directory: %s' % self.path)

        if os.path.splitext(os.path.realpath(path))[1] != '.app':
            raise ApplicationError('Not an application bundle: %s' % path)

    def __repr__(self):
        return self.path

    @property
    def info(self):
        if not self.__cached_info:
            self.__cached_info = ApplicationInfo(self)
        return self.__cached_info

    @property
    def version(self):
        info = self.info

        for k in ('version', 'short_version'):
            value = getattr(info,k)
            if value is not None:
                return value

        return 'UNKNOWN'

class ApplicationInfo(dict):
    """
    Information for an application, as dictionary
    """
    def __init__(self,application):
        self.path = os.path.join(application.path,'Contents','Info.plist')

        if not os.path.isfile(self.path):
            raise ApplicationError('No such file: %s' % self.path)

        try:
            self.update(plistlib.readPlist(self.path).items())
        except ExpatError,emsg:
            raise ApplicationError('Error parsing %s: %s' % (self.path,emsg))

    def __repr__(self):
        return unicode('%s %s' % (self.name,self.version))

    def __getattr__(self,attr):
        if attr in INFO_BUNDLE_NAME_MAP.keys():
            attr = INFO_BUNDLE_NAME_MAP[attr]

        try:
            return self[attr]
        except KeyError:
            return None

class ApplicationTree(list):
    """
    Tree of OS/X applications (.app directory bundles)
    """
    def __init__(self,path='/Applications',max_depth=2):
        self.path = path
        self.max_depth = max_depth

        self.update()

    def update(self):
        """
        Update application tree recursively with load_tree()
        """

        def load_tree(path,depth=0):
            """
            Load tree items from given path
            """
            if depth >= self.max_depth:
                return []

            apps = []
            for s in sorted(os.path.join(path,d) for d in os.listdir(path)):
                if not os.path.isdir(s):
                    continue

                if os.path.splitext(s)[1][1:] == 'app':
                    apps.append(Application(s))
                else:
                    apps.extend(load_tree(s,depth=depth+1))

            return apps

        if not os.path.isdir(self.path):
            return

        list.__delslice__(self,0,len(self))
        self.extend(load_tree(self.path))




from datetime import datetime
from systematic.log import LogFile, LogEntry, LogFileError

class BackupTask(object):
    def __init__(self, entry):
        self.backup_date = None
        self.started = None
        self.finished = None
        self.state = 'UNKNOWN'
        self.messages = []

        self.add_logentry(entry)

    def __repr__(self):
        if self.state == 'finished':
            return 'FINISHED %s' % self.backup_date
        elif self.state == 'failed':
            return 'FAILED %s' % self.started
        else:
            return '%s %s' % (self.state.upper(), self.started)

    def add_logentry(self, entry):
        if entry.message == 'Starting automatic backup':
            self.started = entry.time
            self.state = 'started'

        elif entry.message[:13] == 'Backup failed':
            self.finished = entry.time
            self.state = 'failed'

        elif entry.message[:19] == 'Created new backup:':
            self.backup_date = datetime.strptime(entry.message[19:].strip(), '%Y-%m-%d-%H%M%S')
            self.state = 'created'

        elif entry.message == 'Starting post-backup thinning':
            self.state = 'cleanup'

        elif entry.message[:30] == 'Post-backup thinning complete:':
            self.stateu = 'cleanup_finished'

        elif entry.message == 'Backup completed successfully.':
            self.finished = entry.time
            self.state = 'finished'

        self.messages.append(entry)


class BackupdLogEntry(LogEntry):
    pass

class BackupdLog(LogFile):
    lineloader = BackupdLogEntry

    def __init__(self):
        LogFile.__init__(self, '/var/log/system.log')
        self.backups = []

    def __is_backupd_logentry__(self, entry):
        return entry.program == 'com.apple.backupd'

    @property
    def last_failed_backup(self):
        try:
            return [x.finished for x in self.backups if x.state == 'failed'][-1]
        except IndexError:
            return None

    @property
    def last_successful_backup(self):
        try:
            return [x.backup_date for x in self.backups if x.state == 'finished'][-1]
        except IndexError:
            return None

    def next(self):
        entry = self.next_iterator_match(iterator='default', callback=self.__is_backupd_logentry__)
        if entry is None:
            return None

        if entry.message == 'Starting automatic backup':
            self.backups.append(BackupTask(entry))
        elif self.backups:
            self.backups[-1].add_logentry(entry)
        return entry

if __name__ == '__main__':
    log = BackupdLog()
    log.reload()
    print log.last_failed_backup
    print log.last_successful_backup



#!/usr/bin/env python
"""
Wrapper for scons build systems to install dependencies from homebrew
build system on OS/X.

This is only meant to allow automatic tool installs and not at all to
replace or replicate all functionality of direct use of brew command.

Example usage:
brew = Homebrew()

# Example how to list installed packages
for package in brew.values():
    print package.name, package.versions

# Example how to list available packages
for name in brew.available_formulas():
    print name

# Example how to update homebrew system
brew.update()
brew.upgrade_all()
brew.cleanup()

# Example how to install packages
for arg in sys.argv[1:]:
    try:
        brew.install(arg)
    except HomebrewError,emsg:
        print emsg

"""

import os
from subprocess import Popen, PIPE

HOMEBREW_PREFIX = '/usr/local'
HOMEBREW_FORMULAS = os.path.join(HOMEBREW_PREFIX, 'Library', 'Formula')
HOMEBREW_DEFAULT_COMMAND = os.path.join(HOMEBREW_PREFIX, 'bin', 'brew')

class HomebrewError(Exception):
    pass

class Homebrew(dict):
    """
    Wrapper for homebrew packaging system's brew command
    """
    def __init__(self, brew=HOMEBREW_DEFAULT_COMMAND):
        self.brew = brew
        self.update_installed()

    def keys(self):
        """
        Return package names as sorted list
        """
        return sorted(dict.keys(self))

    def items(self):
        """
        Return name, package as sorted by name with self.keys()
        """
        return [(k, self[k]) for k in self.keys()]

    def values(self):
        """
        Return packages as sorted by name with self.keys()
        """
        return [self[k] for k in self.keys()]

    def update_installed(self):
        """
        Update our list of installed homebrew package names
        """
        self.clear()
        cmd = [self.brew, 'list']
        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        (stdout, stderr) = p.communicate()
        for l in filter(lambda x: x.strip()!='',  stdout.split('\n')):
            self[l] = HomebrewPackage(self, l)

    def available_formulas(self):
        """
        List available packages with Homebrew formula
        """
        return [os.path.splitext(x)[0] for x in filter(lambda x:
            x[-3:]=='.rb', os.listdir(HOMEBREW_FORMULAS)
        )]

    def cleanup(self):
        """
        Run brew clean to remove stale files
        """
        cmd = [self.brew, 'cleanup']
        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        (stdout, stderr) = p.communicate()
        if p.returncode != 0:
            raise HomebrewError('Error cleaning up homebrew: %s' % stderr)
        return stdout

    #noinspection PyMethodOverriding
    def update(self):
        """
        Update homebrew package descriptions with brew update
        """
        cmd = [self.brew, 'update']
        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        (stdout, stderr) = p.communicate()
        if p.returncode != 0:
            raise HomebrewError('Error updating homebrew\n%s' % stdout)
        return stdout

    def upgrade_all(self):
        """
        Run brew upgrade --all to upgrade installed packages. Running
        self.cleanup() afterwards is recommended to get rid of old versions.
        """
        cmd = [self.brew, 'upgrade', '--all']
        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        (stdout, stderr) = p.communicate()
        if p.returncode != 0:
            raise HomebrewError('Error upgrading brew packages\n%s' % stdout)
        return stdout

    def is_installed(self, package):
        """
        Returns if package is installed,  at least some version
        """
        return package in self.keys()

    def install(self, package, force_install=False):
        """
        Install package if formula is available
        """
        if not force_install and self.is_installed(package):
            return
        if not package in self.available_formulas():
            raise HomebrewError('Homebrew: unknown package: %s' % package)

        package = HomebrewPackage(self, package)
        package.install()
        self[package.name] = package

class HomebrewPackage(object):
    """
    Abstraction for one homebrew package, with multiple possible versions
    """
    def __init__(self, brew, name):
        self.brew = brew
        self.name = name
        self.cellar = os.path.join(HOMEBREW_PREFIX, 'Cellar', name)

    def __repr__(self):
        return '%s in %s' % (self.name, self.cellar)

    def __getattr__(self, attr):
        if attr == 'versions':
            return os.listdir(self.cellar)

        raise AttributeError('No such HomebrewPackage attribute: %s' % attr)

    def install(self, force_install=False):
        """
        Install a package from homebrew
        """
        if os.path.isdir(self.cellar) and not force_install:
            return

        cmd = [self.brew.brew, 'install', self.name]
        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        (stdout, stderr) = p.communicate()

        if p.returncode != 0:
            raise HomebrewError('Homebrew: error installing %s\n%s' % (self.name, stdout))

        return stdout, stderr



#!/usr/bin/env python
"""
Classes to handle AFP mountpoints, mounting and configuring them.
"""

import os
import re

from configobj import ConfigObj
from subprocess import call, CalledProcessError

re_mountpoint = re.compile(r'^/Volumes/[A-Za-z0-9_-]*$')

DEFAULT_CONFIG_PATH = os.path.join(os.getenv('HOME'), '.afpshares.conf')

class AFPShareError(Exception):
    """
    Exceptions raised when accessing AFP shares
    """
    def __str__(self):
        return self.args[0]

class AFPShareConfig(dict):
    """
    Reader for AFP share configuration files.
    """
    def __init__(self, config_path=DEFAULT_CONFIG_PATH):
        dict.__init__(self)
        self.config = None
        self.path = config_path
        if not os.path.isfile(self.path):
            raise AFPShareError('No such file: %s' % self.path)
        if not os.access(self.path, os.R_OK):
            raise AFPShareError(
                'No permission to read configuration: %s' % self.path
            )

        self.config = ConfigObj(self.path)
        if self.config.has_key('Options'):
            self.options = dict(self.config['Options'])
        else:
            self.options = {}
        for k in filter(lambda k: k!='Options', self.config.keys()):
            try:
                self[k] = AFPShareDisk(k, self.config[k])
            except AFPShareError, e:
                print e
                continue

    def __getattr__(self, attr):
        if attr == 'disks':
            return self.values()
        try:
            return self.options[attr]
        except KeyError:
            pass
        raise AttributeError('No such AFPShareConfig attribute: %s' % attr)

    def __getitem__(self, item):
        try:
            return self[item]
        except KeyError:
            pass
        raise KeyError('No such AFP share: %s' % item)

class AFPShareDisk(dict):
    """
    AFP network share disk specification
    """
    def __init__(self, name, settings):
        dict.__init__(self)
        self.name = name
        self.update(**{
            'address': None,
            'path':None,
            'mountpoint':None,
            'username': None,
            'password': None,
        })

        for k, v in settings.items():
            if not self.has_key(k):
                raise AFPShareError('Unsupported disk option: %s' % k)
            self[k] = v
        for k in ['address', 'path', 'mountpoint']:
            if self[k] is None:
                raise AFPShareError('Missing required option %s' % k)

        if self.has_key('username') and not self.has_key('password'):
            raise AFPShareError('Username defined but no password given')

        if self.has_key('password') and not self.has_key('username'):
            raise AFPShareError('Password defined but no username given')

        if not re.match(re_mountpoint, self.mountpoint):
            raise AFPShareError(
                'Unsupported mountpoint path: %s' % self.mountpoint
            )

    def __getattr__(self, attr):
        if attr == 'afp_path_nopass':
            #noinspection PyStringFormat
            return 'afp://%(address)s%(path)s' % self
        if attr == 'afp_path':
            if self.username is not None:
                #noinspection PyStringFormat
                return 'afp://%(username)s:%(password)s@%(address)s%(path)s' % self
            #noinspection PyStringFormat
            return 'afp://%(address)s%(path)s' % self
        try:
            return self[attr]
        except KeyError:
            pass
        raise AttributeError('No such %s attr: %s' % (self.name, attr))

    def __str__(self):
        return '%s: %s mounted on %s' % (
            self.name,  self.afp_path_nopass, self.mountpoint
        )

    def status(self):
        """
        Return status of mountpoint as string:
        'not mounted', 'mounted by other user', 'mounted by myself'
        """
        if not os.path.ismount(self.mountpoint):
            return 'not mounted'
        if not os.access(self.mountpoint, os.W_OK):
            return 'mounted by other user'
        else:
            return 'mounted by myself'

    def mount(self):
        """
        Mount the AFP share
        """
        if os.path.ismount(self.mountpoint):
            raise AFPShareError('Already mounted: %s' % self.mountpoint)

        if not os.path.isdir(self.mountpoint):
            try:
                os.makedirs(self.mountpoint)
                if os.stat(self.mountpoint).st_uid != os.getuid():
                    os.chown(self.mountpoint, os.getpid(), os.getgid())
            except IOError, (ecode, emsg):
                raise AFPShareError(emsg)
            except OSError, (ecode, emsg):
                raise AFPShareError(emsg)

        try:
            call(['mount_afp', self.afp_path, self.mountpoint])
        except CalledProcessError, emsg:
            raise AFPShareError(emsg)

    def umount(self):
        """
        Umount the AFP share
        """
        if not os.path.ismount(self.mountpoint):
            return
        if not os.access(self.mountpoint, os.W_OK):
            raise AFPShareError('No write access to %s' % self.mountpoint)
        try:
            call(['hdiutil', 'detach', self.mountpoint])
        except CalledProcessError, emsg:
            raise AFPShareError(emsg)




#!/usr/bin/env python
"""
Module to create application dictionaries for appscript using command line sdef tool
"""

import os
from subprocess import check_output
from lxml import etree as ET

class SDEFError(Exception):
    def __str__(self):
        return self.args[0]

class SDEF(object):
    def __init__(self,path):
        self.path = path
        if not os.path.isdir(path):
            raise SDEFError('Not a directory: %s' % path)
        self.tree = ET.fromstring(check_output(['sdef',path]))

    def __getattr__(self,attr):
        if attr == 'terms':
            return self.__generate_terms()

    def __generate_terms(self):
        output = 'version = 1.1\npath = %s' % self.path
        classes = []
        enums = []
        properties = []
        elements = []
        commands = []
        for suite in self.tree.xpath('suite'):
            for node in suite.xpath('class'):
                name = node.get('name').replace(' ','_')
                code = node.get('code')
                classes.append((name,code))
                if node.get('inherits') is None:
                    continue
                element = node.get('plural').replace(' ','_')
                elements.append((element,code))
            for node in suite.xpath('enumeration/enumerator'):
                name = node.get('name').replace(' ','_')
                code = node.get('code')
                enums.append((name,code))
            for node in suite.xpath('class/property'):
                name = node.get('name').replace(' ','_')
                code = node.get('code')
                properties.append((name,code))
            for node in suite.xpath('command'):
                name = node.get('name').replace(' ','_')
                code = node.get('code')
                cparams = []
                for p in node.xpath('parameter'):
                    pname = p.get('name').replace(' ','_')
                    pcode = p.get('code')
                    cparams.append((pname,pcode))
                commands.append((name,code,cparams))

        output += '\nclasses = %s' % classes
        output += '\nenums = %s' % enums
        output += '\nproperties = %s' % properties
        output += '\nelements = %s' % elements
        output += '\ncommands = %s' % commands
        return output



#!/usr/bin/env python
"""
Scripts and system tool wrappers for OS/X

This module is split from darwinist module to platform dependent tool
"""

import sys,os,glob
from setuptools import setup

VERSION='4.1.0'

setup(
    name = 'darwinist',
    keywords = 'System Management Utility OS/X Darwin Scripts',
    description = 'Sysadmin utility modules and scripts for OS/X',
    author = 'Ilkka Tuohela',
    author_email = 'hile@iki.fi',
    version = VERSION,
    url = 'http://tuohela.net/packages/darwinist',
    license = 'PSF',
    scripts = glob.glob('bin/*'),
    packages = (
        'darwinist',
    ),
    install_requires = (
        'systematic>=4.0.2',
        'appscript',
        'pyfsevents'
    ),
)




"""
Module to parse OS/X process listing from ps command output
"""

import os
import decimal
from subprocess import Popen, PIPE

PS_COMMAND = ['ps', 'auxwww']
PS_FIELDS = [
    'username',
    'pid',
    'cpu_pct',
    'mem_pct',
    'vsz',
    'rss',
    'tty',
    'stat',
    'started',
    'time',
    'command'
]

class ProcessList(list):
    """
    List of processes based on ps output
    """
    def __init__(self, command=PS_COMMAND, fields=PS_FIELDS):
        self.command = command
        self.fields = fields
        self.update()

    def update(self):
        """
        Update list of processes
        """
        list.__delslice__(self, 0, len(self))
        cmd = self.command
        p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        (stdout, stderr) = p.communicate()
        for l in stdout.split('\n')[1:]:
            if l.strip()=='': continue
            self.append(Process(l, fields=self.fields))

    def sort_by_field(self, field, reverse=False):
        """
        Sort processes in-list by given field.
        If reverse is True, the in-line ordering is reversed after
        sorting.
        """
        self.sort(lambda x, y: cmp(x[field], y[field]))
        if reverse:
            self.reverse()

    def filter_user(self, username):
        """
        Filter processes to matching username
        """
        return filter(lambda p: p.username==username, self)

    def filter_command(self, command):
        """
        Filter processes to matching command name: the command name
        is first space separate part from 'command' column and thus
        this doesn't work for matching commands with spaces.
        """
        return filter(lambda p:
            p.command is not None and \
            os.path.basename(p.command.split(None, 1)[0]) == command,
            self
        )

class Process(dict):
    """
    Wrapper class for information for one process
    """
    def __init__(self,line,fields):
        rest = line
        for k in fields[:-1]:
            try:
                value,rest = [x.strip() for x in rest.split(None,1)]
            except ValueError:
                self[k] = rest
                break
            self[k] = value

        if rest.strip() != '':
            self[fields[-1]] = rest.lstrip()

        for k in fields:
            if not self.has_key(k):
                self[k] = None

        for k in ['pid', 'vsz', 'rss']:
            self[k] = int(self[k])

        for k in ['cpu_pct', 'mem_pct']:
            self[k] = decimal.Decimal(self[k])

    def __getattr__(self, attr):
        try:
            return self[attr]
        except KeyError:
            raise AttributeError('No such Process attribute: %s' % attr)



#!/usr/bin/env python
"""
Module for Apple OS/X airport status command access from python
"""
import os
from subprocess import check_output, CalledProcessError

AIRPORT_BINARY = '/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport'

class AirportError(Exception):
    """
    Exception raised by errors processing AirPort status
    """
    def __str__(self):
        return self.args[0]

class AirportStatus(dict):
    """
    Class to call the system 'airport' command.
    """
    def __init__(self):
        if not os.path.isfile(AIRPORT_BINARY):
            raise AirportError('No such command: %s' % AIRPORT_BINARY)

    def __repr__(self):
        self.probe()
        return '%(BSSID)s %(SSID)s channel %(channel)s %(agrCtlRSSI)s dB' % self

    def __getattr__(self, attr):
        try:
            if not self.keys():
                self.probe()
            return self[attr]

        except KeyError:
            raise AttributeError('No such AirportStatus attribute: %s' % attr)

    def probe(self):
        """
        Probe airport status
        """
        self.clear()
        try:
            data = check_output([AIRPORT_BINARY, '-I'])
        except CalledProcessError, emsg:
            print emsg
            return

        for l in data.split('\n'):
            if l.strip() == '':
                continue

            try:
                (key, value) = map(lambda x: x.strip(),  l.split(':', 1))
                self[key] = value
            except ValueError:
                raise AirportError('Error parsing line: %s' % l)

        for k in ['BSSID']:
            if not self.has_key(k):
                continue

            self[k] = ':'.join(['%02x'.upper() % int(x, 16) for x in self[k].split(':')])

    def proximity(self):
        """
        Return proximity of base stations based on signal levels
        """
        headers = ['SSID', 'BSSID', 'RSSI', 'CHANNEL', 'HT', 'CC', 'SECURITY']
        aps = []

        try:
            data = check_output([AIRPORT_BINARY, '-s', self.SSID])
        except CalledProcessError, emsg:
            print emsg
            return

        for l in data.split('\n'):
            if l.strip() == '':
                continue
            l = l.rstrip()
            if headers[:5] == [x.strip() for x in l.split()][:5]:
                continue

            ssid =  l[:32].lstrip()
            bssid = l[33:50].strip("'").upper()
            rssi =  int(l[51:55].strip())
            channel = int(l[56:58].strip())
            aps.append({
                'SSID': ssid, 'BSSID': bssid, 'RSSI': rssi, 'CHANNEL': channel
            })

        aps.sort(lambda x,y: cmp(y['RSSI'], x['RSSI']))
        return aps

